#! /usr/bin/perl

#   	    Make images from PostScript definition

#   	       by John Walker  --  November 2003
#   	      This program is in the public domain.

    use warnings;
    $psfile = 'noEU.ps'; 	    	# PostScript file to render
    $oname = 'noEU'; 	    	# Base name for images generated
    $resolution = 1600;	    	    	# Resolution in dots per inch
    	    	    	    	    	# Careful!  Disc space required for processing
					# goes as the *square* of the resolution.  At 1600 DPI,
					# you need about 1 Gb to hold the intermediate full
					# resolution images plus almost half as much RAM for the
					# cropping and flipping phases.

    #	The following variables allow you to divide the process
    #	of generating the image into three consecutive phases:
    #
    #	    Phase 1:  Rendering PostScript into high-resolution
    #	    	      bitmap and rough cropping.
    #
    #	    Phase 2:  Fine cropping and rotation.
    #
    #	    Phase 3:  Scaling to desired sizes and PNG image
    #	    	      generation.
    #
    #	Dividing the processing into phases allows it to be split
    #	across different machines depending on their capability
    #	or, when debugging, to keep a known good output from an
    #	earlier phase when debugging a subsequent one.  Given the
    #	resources required when rendering with very high spatial
    #	resolution, this can save a lot of time.  If you prefer
    #	to just run straight through, simply set $phase1, $phase2,
    #	and $phase3 all to 1.
   $phase = 1;
   $phase1 = $phase == 1;
   $phase2 = $phase == 2;
   $phase3 = $phase == 3;
    #	Use the following line for monolithic processing
    $phase1 = $phase2 = $phase3 = 1;
					
    #	You may be tempted to "optimise" this program by combining
    #	NetPBM commands run here file-to-file into pipelines.  They're
    #	split out for a reason!  Some NetPBM utilities write their input
    #	into a temporary file if fed from standard input.  This temporary
    #	file may go onto a filesystem with less space than the one where
    #	you're preparing the images, resulting in out of memory errors
    #	even though you have abundant space in the area in which you're
    #	working.  Further, running multiple commands in a pipeline means
    #	the memory used by all of them is required simultaneously, which
    #	can lead to thrashing or crashes due to swap file exhaustion.
    #	You're only going to run this a few times--who cares how time
    #	efficient it is?
    
    @imgres = (640, 768, 1024, 1200, 1600, 2048, 4096, 8192);   # Desired antialised resolutions

if ($phase1) {
    #	Generate master monster-resolution image.  This is
    #	the raw material for subsequent image generation.
    
    $cmd = "echo quit | gs -q -dNOPAUSE -r${resolution}x${resolution} " .
    	   "-sOutputFile=- -sDEVICE=ppmraw $psfile >makeImage.tmp";
    &syscmd($cmd);

    $s = `pnmfile makeImage.tmp`;
    $s =~ m/,\s(\d+)\sby\s(\d+)\s/;
}

if ($phase2) {
    
    #	Perform a precise crop on the image.  We do this in
    #	a separate step since pnmcrop does not require the
    #	entire image to be read into memory when its input is
    #	a file, which permits it to run on configurations
    #	with less memory than the size of the uncropped image.
    
    &syscmd("pnmcrop -white makeImage.tmp >makeImage1.tmp");
    &syscmd("rm makeImage.tmp");
    &syscmd("mv makeImage1.tmp makeImage.tmp");
}

if ($phase3) {
    
    #	Determine width of cropped image
    
    $s = `pnmfile makeImage.tmp`;
    $s =~ m/,\s(\d+)\sby\s(\d+)\s/;
    $cwid = $1;
    
    #	Create comments for text chunk in the PNG file
    
    open(TC, ">makeImage2.tmp") || die("Cannot create makeImage2.tmp");
    print TC << "EOD";
Title	    No EU
Author	    John Walker (http://www.fourmilab.ch/)
Date	    December 2003
Source	    http://www.fourmilab.ch/evilempire/noEU/
Note	    This image is available in a variety of resolutions
    	    and as an Encapsulated PostScript file at the Source
	    Web page.
Copyright   This image is in the public domain
EOD
    close(TC);
    
    #	Iterate over requested image resolutions (expressed as a width)
    #	and generate scaled images for each.
    
    for ($i = 0; $i <= $#imgres; $i++) {
    	$w = $imgres[$i];
	    
	#   Compute pad adequate to antialias an image of this size
	$pad = int(($cwid / $w) + 0.5);
    
	&syscmd("pnmmargin -white $pad makeImage.tmp | pnmscale -width $w | pnmtopng -ztxt makeImage2.tmp -interlace -compression 9 >${oname}_$w.png");
	$s = `pngtopnm ${oname}_$w.png | pnmfile`;
	chomp($s);
	$s =~ s/^[^,]*,\s//;
	print("${oname}_$w.png: $s\n");
    }

    #	Generate full-resolution monster image (no anti-aliasing, and hence no pad needed)
    
    $s = `pnmfile makeImage.tmp`;
    $s =~ m/,\s(\d+)\sby\s(\d+)\s/;
    $w = $1;
    &syscmd("pnmtopng -ztxt makeImage2.tmp -interlace -compression 9 makeImage.tmp >${oname}_$w.png");
    $s = `pngtopnm ${oname}_$w.png | pnmfile`;
    chomp($s);
    $s =~ s/^[^,]*,\s//;
    print("${oname}_$w.png: $s\n");
    
    #	Take out the trash
    
    &syscmd("rm -f makeImage.tmp makeImage2.tmp");
}
    
    #	Echo and execute a system command
    sub syscmd {
    	my ($cmd) = @_;
	
	print(STDERR "$cmd\n");
	system($cmd);
    }
